package speechNlu

import (
	"encoding/json"
	"fmt"
	"log"
	"speech-nlu-parse/global"
	"speech-nlu-parse/pkg/logger"
	"time"

	amqp "github.com/rabbitmq/amqp091-go"
)

type Consumer struct {
	conn    *amqp.Connection
	channel *amqp.Channel
	tag     string
	done    chan error
}

func failOnError(err error, msg string) {
	if err != nil {
		global.Logger.Errorf("%s: %s", msg, err)
		return
	}
}

var consumerErr int

func NewConsumer(amqpURI, exchangeName, exchangeType string) (*Consumer, error) {
	c := &Consumer{
		conn:    nil,
		channel: nil,
		done:    make(chan error),
	}

	var err error

	// 最大重试次数
	maxRetries := 3

	// 连接 RabbitMQ
	log.Printf("dialing %q", amqpURI)
	c.conn, err = amqp.Dial(amqpURI)
	if err != nil {
		global.Logger.Errorf("Dial: %s", err)
		consumerErr++
		if consumerErr > maxRetries {
			return nil, fmt.Errorf("failed to connect after %d retries", maxRetries)
		}
		time.Sleep(3 * time.Second)
		return NewConsumer(amqpURI, exchangeName, exchangeType) // 重试连接
	}
	defer c.conn.Close()
	// 创建信道
	log.Printf("got Connection, getting Channel")
	c.channel, err = c.conn.Channel()
	if err != nil {
		global.Logger.Errorf("Failed to open a channel: %s", err)
		return nil, err
	}
	defer c.channel.Close()

	// 声明交换机
	log.Printf("got Channel, declaring Exchange (%q)", exchangeName)
	err = c.exchangeDeclare(exchangeName, exchangeType)
	if err != nil {
		global.Logger.Errorf("Failed to declare exchange: %s", err)
		return nil, err
	}

	queue := "news"
	routingKey := "news"

	// 声明队列并绑定
	log.Printf("declared Exchange, declaring Queue %q", queue)
	err = c.queueDeclare(queue)
	if err != nil {
		global.Logger.Errorf("Failed to declare queue %s: %s", queue, err)
		return nil, err
	}

	err = c.channel.QueueBind(queue, routingKey, exchangeName, false, nil)
	if err != nil {
		global.Logger.Errorf("Failed to bind queue %s: %s", queue, err)
		return nil, err
	}

	// 设置 QoS
	err = c.channel.Qos(10, 0, false)
	if err != nil {
		global.Logger.Errorf("Failed to set QoS: %s", err)
		return nil, err
	}

	// 启动消费者
	err = c.Consume()
	if err != nil {
		global.Logger.Errorf("Failed to start consumer: %s", err)
		return nil, err
	}

	// 监听通道关闭事件
	closeChan := make(chan *amqp.Error, 1)
	notifyClose := c.channel.NotifyClose(closeChan)

	// 处理通道关闭事件
	for {
		select {
		case e := <-notifyClose:
			global.Logger.Errorf("chan通道错误：%v", e.Error())
			time.Sleep(5 * time.Second)
			log.Println("休息5秒后重新连接")

			// 重新连接
			c, err = NewConsumer(amqpURI, exchangeName, exchangeType)
			if err != nil {
				global.Logger.Errorf("Failed to reconnect: %s", err)
				return nil, err
			}
			return c, nil // 返回新的 Consumer 实例
		}
	}
}

func (c *Consumer) queueDeclare(queue string) error {
	_, err := c.channel.QueueDeclare(queue, true, false, false, false, nil)
	return err
}

func (c *Consumer) exchangeDeclare(exchangeName, exchangeType string) error {
	err := c.channel.ExchangeDeclare(
		exchangeName, // name of the exchange
		exchangeType, // type
		true,         // durable
		false,        // delete when complete
		false,        // internal
		false,        // noWait
		nil,          // arguments
	)
	return err
}

func (c *Consumer) Shutdown() error {
	// will close() the deliveries channel
	if err := c.channel.Cancel(c.tag, true); err != nil {
		return fmt.Errorf("Consumer cancel failed: %s", err)
	}

	if err := c.conn.Close(); err != nil {
		return fmt.Errorf("AMQP connection close error: %s", err)
	}

	defer log.Printf("AMQP shutdown OK")

	// wait for handle() to exit
	return <-c.done
}

type ReData struct {
	Cols []string      `json:"cols"`
	Dat  []interface{} `json:"dat"`
}

func (c *Consumer) Consume() {
	msgs, err := c.channel.Consume("news", "", false, false, false, false, nil)
	failOnError(err, "Failed to register a consumer")
	for d := range msgs {
		resp := ReData{}
		err := json.Unmarshal(d.Body, &resp)
		if err != nil {
			global.Logger.WithFields(logger.Fields{"requestId": requestId}).Errorf("subdev json.Unmarshal error: %v", err)
			d.Nack(false, true)
			break
		}

		global.Logger.WithFields(logger.Fields{
			"requestId":  requestId,
			"RoutingKey": d.RoutingKey,
			"Body":       fmt.Sprintf("%s", d.Body),
			"re":         re,
		}).Info("news")

		// 初始化变量
		var (
			state, offset int
			mediaId, ip   string
		)
		//todo：还需要判断domain

		// 遍历 cols 找到索引并取值
		for i, col := range resp.Cols {
			switch col {
			case "state":
				if val, ok := resp.Dat[i].(float64); ok {
					state = int(val)
				}
			case "mediaId":
				if val, ok := resp.Dat[i].(string); ok {
					mediaId = val
				}
			case "offset":
				if val, ok := resp.Dat[i].(float64); ok {
					offset = int(val)
				}
			}
			//todo:还需要ip和mac
			feedBack, err := FeedBackNews(state, offset, ip, mediaId)
			failOnError(err, "FeedBackNews err")
			if feedBack.Code != 0 {
				d.Nack(false, true)
				break
			}
			d.Ack(false)
		}

	}
}
